#!/usr/bin/env python3


import argparse
import sys
import yaml

from schema_validator import validation
import logging as LOG

LOG.basicConfig(level=LOG.INFO, datefmt='%Y-%m-%d %H:%M:%S',
                format='%(asctime)-15s - [%(levelname)s] %(module)s:%(lineno)d: '
                       '%(message)s', )


def _make_argument_parser():
    parser = argparse.ArgumentParser(prog='schema-validator')
    parser.add_argument(
        '--schema',
        help='Path to the JSON-like schema file used for validation',
    )
    parser.add_argument(
        '--data',
        help='Path to the YAML file to validate',
    )
    return parser


def main():
    parser = _make_argument_parser()
    args = parser.parse_args()

    yaml_file = args.data
    schema_file = args.schema

    LOG.info(f'Loading data file {yaml_file}')
    with open(yaml_file) as fd:
        data = yaml.safe_load(fd)

    LOG.info(f'Loading schema file {schema_file}')
    with open(schema_file) as fd:
        schema = yaml.safe_load(fd)

    try:
        result = validation.validate(schema=schema, data=data)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(1)


if __name__ == '__main__':
    main()
