#!/usr/bin/env python3

import logging as LOG

import jsonschema

LOG.basicConfig(level=LOG.INFO, datefmt='%Y-%m-%d %H:%M:%S',
                format='%(asctime)-15s - [%(levelname)s] %(module)s:%(lineno)d: '
                       '%(message)s', )


# TODO add custom exception, to be able correctly parse traceback in
# Ansible Module

def validate(**kwargs):
    schema = kwargs.pop('schema')
    data = kwargs.pop('data')
    try:
        validator = jsonschema.Draft4Validator(schema)
        validator.check_schema(schema)
        LOG.info('Schema itself looks valid')
    except jsonschema.exceptions.SchemaError as ex:
        LOG.error('Schema itself is invalid')
        raise Exception(ex.message + '\n' + str(ex.absolute_path))
    try:
        jsonschema.validate(instance=data, schema=schema)
    except jsonschema.ValidationError as ex:
        if len(ex.path) > 0:
            detail = (f'Invalid input for field/attribute {ex.path.pop()}.\n'
                      f'Value: {ex.instance} => {ex.message}')
        else:
            detail = ex.message
        raise Exception(detail)
    except TypeError as ex:
        # NOTE: If passing non string value to patternProperties parameter,
        #       TypeError happens. Here is for catching the TypeError.
        import six
        detail = six.text_type(ex)
        raise Exception(f'TypeError: {detail}')
    LOG.info('Data seems valid for schema\n')
    return {'message': 'validation ok'}
